/*  @(#)04  1.4 src/com/tivoli/pd/jazn/demo/PDDemoSetup.java, pd.jazn, am611 07/03/15 14:13:26 @(#)  */

/*
 *
 *
 * Licensed Materials - Property of IBM
 * 5748-XX8
 * (c) Copyright International Business Machines Corp. 2001, 2002
 * All Rights Reserved.
 * US Government Users Restricted Rights - Use, duplication or disclosure
 * restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 * File Name: PDDemoSetup.java
 */


import com.tivoli.pd.jadmin.*;
import com.tivoli.pd.jutil.*;

import java.io.*;
import java.net.*;
import java.security.*;
import java.util.*;

/**
 * Access Manager PDDemoSetup class.
 *
 * <p>
 * This class creates the configuration file necessary to run either the
 * PDPermissionDemo or the PDJaasDemo. The user is given the option of
 * creating/updating the files in the lib/security directory of the jre
 * currently in use to which the changes necessary to support the
 * PDJaasDemo.
 *
 * <p>
 * The PDDemoSetup prompts the user for the information needed to create
 * a configuration file. If the creation of the configuration file fails,
 * the user is prompted for whether or not to try again.
 *
 * <p>
 * The PDDemoSetup prompts the user for whether or not the user wishes to
 * setup the jre to run the PDJaasDemo. The existing files in the
 * lib/security directory of the jre are backed up prior to update.
 *
 * @author      Douglas Brown
 *
 */
public final class PDDemoSetup
{

    private final String PDDemoSetup_java_sourceCodeID="$Id: @(#)04  1.4 src/com/tivoli/pd/jazn/demo/PDDemoSetup.java, pd.jazn, am611 07/03/15 14:13:26 @(#) $";

    private static String adminId_  = "sec_master";
    private static String adminPwd_ = null;
    private static String appId_    = null;
    private static String host_ = null;
    private static String mode_ = null;
    private static String port_ = null;
    private static String policysvrHost_ = null;
    private static String policysvrPort_ = "7135";
    private static String authzsvrHost_ = null;
    private static String authzsvrPort_ = "7136";
    private static String domain_ = "Default";
    private static String directory_ = null;
    private static String again_ = "Y";

    /**
     * Prompts for input.
     *
     * <p>
     * @param reader  the reader used to retrieve input.
     *
     * @param prompt  string used for prompt when retrieving input.
     *
     * @param def     string default returned when no input is provided.
     *
     * <p>
     * @return        string input.  This is the value specified by the default
     *                string when no input is provided.
     */
    private static String prompt(BufferedReader reader,
                                 String prompt,
                                 String def) throws java.io.IOException
    {
        String result = null;
        System.out.print(prompt);

        if (def != null)
        {
            System.out.print(" [");
            System.out.print(def);
            System.out.print("]");
        }

        System.out.print(":  ");
        result = reader.readLine();
        if (result.length() == 0)
        {
            result = def;
        }
        return result;
    }

    /**
     * Constructs a PDDemoSetup object.
     */
    public PDDemoSetup(String [] argv)
    {}

    /**
     * Main program.
     */
    public static void main (String [] argv)
    {

        PDMessages msgs = null;
        URL configURL = null;
        String config = null;
        String keystore = null;
        String jaas = "Y";
        ArrayList plcyServers = null;
        ArrayList authzServers = null;

        String fs = File.separator;
        boolean notComplete = true;
        boolean noError = true;

        // construct reader from standard input stream.
        InputStreamReader input = new InputStreamReader(System.in);
        BufferedReader rdr = new BufferedReader(input);

        // create the PDMessages output parameter
        msgs = new PDMessages();

        // Initialize the PDAdmin API.
        try
        {
            PDAdmin.initialize("ItimJunctionUtil", msgs);
            processMsgs(msgs);
        }
        catch (PDException e)
        {
            System.out.println("PDException occurred while initializing PDAdmin.");
            handlePDException(e);
            return;
        }

        // define the locale
        Locale locale = Locale.getDefault();

        // loop until either completed or exit requested
        do {
            //reset the noError flag
            noError = true;

            try {

                // get input data needed
                String adminId  = prompt(rdr, "Administrator user ID", adminId_);
                adminId_ = adminId;

                String adminPwd = prompt(rdr, "Administrator password", adminPwd_);
                adminPwd_ = adminPwd;

                String appId    = prompt(rdr, "Application Server name", appId_);
                appId_ = appId;

                String host = prompt(rdr, "Application Server host name", host_);
                host_ = host;

                String mode = prompt(rdr, "Application Sever mode (local|remote)", mode_);
                mode_ = mode;

                String port = prompt(rdr, "Application Server TCP/IP port", port_);
                port_ = port;

                String policysvrHost = prompt(rdr, "Policy Server host name", policysvrHost_);
                policysvrHost_ = policysvrHost;

                String policysvrPort = prompt(rdr, "Policy Server port", policysvrPort_);
                policysvrPort_ = policysvrPort;

                String authzsvrHost = prompt(rdr, "Authentication Server host name", authzsvrHost_);
                authzsvrHost_ = authzsvrHost;

                String authzsvrPort = prompt(rdr, "Authentication Server port", authzsvrPort_);
                authzsvrPort_ = authzsvrPort;

                String domain = prompt(rdr, "Access Manager domain", domain_);
                domain_ = domain;

                String directory = prompt(rdr, "Directory for configuration file", directory_);
                directory_ = directory;

            } catch (Exception e) {
                System.out.println("Exception occurred.");
                e.printStackTrace();
                noError = false;
            }

            if (noError) {

                // create the configuration file and keystore file paths
                config = directory_ + fs + "configfile";
                keystore = directory_ + fs + "keystore";

                // see if the configuration file or keystore file already exists
                boolean foundIt = true;
                try
                {
                    FileReader fr = new FileReader(config);
                    fr = new FileReader(keystore);
                }
                catch (FileNotFoundException e)
                {
                  foundIt = false;
                }
                if (foundIt) {
                    System.out.println("Configfile already exists.");
                    noError = false;
                }
            }

            if (noError) {

                // create the ArrayList of policy server information
                plcyServers = new ArrayList();
                try
                {
                    PDSvrInfo pSvr = new PDSvrInfo(policysvrHost_,
                                                   Integer.parseInt(policysvrPort_),
                                                   1,
                                                   locale,
                                                   msgs);
                    plcyServers.add(pSvr);
                }
                catch (NumberFormatException e)
                {
                    System.out.println("NumberFormatException occurred.");
                    noError = false;
                }
                catch (PDException e)
                {
                    System.out.println("PDException occurred while creating a PDSvrInfo instance for the policy server.");
                    handlePDException(e);
                    noError = false;
                }
            }

            if (noError) {

                // create the ArrayList of authorization server information
                authzServers = new ArrayList();
                try
                {
                    PDSvrInfo aSvr = new PDSvrInfo(authzsvrHost_,
                                                   Integer.parseInt(authzsvrPort_),
                                                   1,
                                                   locale,
                                                   msgs);
                    authzServers.add(aSvr);
                }
                catch (NumberFormatException e)
                {
                    System.out.println("NumberFormatException occurred.");
                    noError = false;

                }
                catch (PDException e)
                {
                    System.out.println("PDException occurred while creating a PDSvrInfo instance for the authentication server.");
                    handlePDException(e);
                    noError = false;

                }
            }

            if (noError) {

                // Configure the Access Manager Application Server.
                try
                {
                    // create the application server specification
                    PDAppSvrSpec appsvrSpec = null;
                    if (mode_.equals("local")) {
                        appsvrSpec = (PDAppSvrSpec) new PDAppSvrSpecLocal(appId_,
                                                                null,
                                                                null,
                                                                null,
                                                                host_,
                                                                Integer.parseInt(port_),
                                                                plcyServers,
                                                                authzServers,
                                                                false,
                                                                0,
                                                                new URL ("file:///" + directory_),
                                                                locale,
                                                                msgs);
                    } else {
                        appsvrSpec = (PDAppSvrSpec) new PDAppSvrSpecRemote(appId_,
                                                                  null,
                                                                  null,
                                                                  null,
                                                                  host_,
                                                                  Integer.parseInt(port_),
                                                                  plcyServers,
                                                                  authzServers,
                                                                  locale,
                                                                  msgs);
                }
                    processMsgs(msgs);

                    // create the configuration file url
                    configURL = new URL ("file:///" + config);

                    // create the keystore file url
                    URL keystoreURL = new URL ("file:///" + keystore);

                    // create a PD Global context
                    PDAppSvrConfig.configureAppSvr(adminId_,
                                                   adminPwd_.toCharArray(),
                                                   domain_,
                                                   appsvrSpec,
                                                   configURL,
                                                   keystoreURL,
                                                   PDAppSvrConfig.PDAPPSVRCFG_CREATE,
                                                   locale,
                                                   msgs);
                    processMsgs(msgs);
                    System.out.println("Configuration file successfully created");
                    notComplete = false;

                }
                catch (PDException e)
                {
                    System.out.println("PDException occurred during configureAppSvr");
                    handlePDException(e);
                    noError = false;
                }
                catch (MalformedURLException e)
                {
                    System.out.println("MalformedURL Exception occurred");
                    e.printStackTrace();
                    noError = false;
                }
            }

            if (!noError) {
                try {
                    String again = prompt(rdr, "Try again", again_);
                    if (again.equalsIgnoreCase("n")) {
                        again_ = "N";
                    }
                } catch (Exception e) {
                    System.out.println("Exception occurred.");
                    e.printStackTrace();
                    again_ = "N";
                }
            }

        } while (notComplete && (again_.equals("Y")));

        // Shutdown PDAdmin
        try
        {
            PDAdmin.shutdown(msgs);
            processMsgs(msgs);
        }
        catch (PDException e)
        {
            handlePDException(e);
            System.out.println("Error while shutting down PDAdmin");
        }

        // Check to see if JAAS setup has been done
        String jre = System.getProperty("java.home");

        File jaasPolicy = new File(jre + fs + "lib" + fs + "security" + fs + "jaas.policy");
        if (jaasPolicy.exists()) {
            jaas = "N";
            System.out.println("Setup necessary for the JAAS demo is already done.");
        } else {

            // Ask if the JAAS setup is required
            try {
                jaas = prompt(rdr, "Do the additional setup necessary for the JAAS demo?", "Y");
            } catch (Exception e) {
                System.out.println("Exception occurred.");
                e.printStackTrace();
                jaas = "N";
            }
        }

        // JAAS demo setup
        if (jaas.equalsIgnoreCase("y")) {

            String filename = null;
            BufferedReader in = null;
            BufferedWriter out = null;

            // back up the java.security file
            try {

                filename = jre + fs + "lib" + fs + "security" + fs + "java.security";
                System.out.println("Backing up file " + filename);

                in = new BufferedReader(new FileReader(filename));
                out = new BufferedWriter(new FileWriter(filename + ".orig"));

                String current = null;
                while ((current = in.readLine()) != null)
                {
                  out.write(current, 0, current.length());
                  out.newLine();
                }
                in.close();
                out.close();

            } catch (Exception e){
                System.out.println("Exception occurred backing up java.security");
            }

            // update the java.security file
            try {

                System.out.println("Updating file " + filename);

                in = new BufferedReader(new FileReader(filename + ".orig"));
                out = new BufferedWriter(new FileWriter(filename));

                String current = null;
                while ((current = in.readLine()) != null)
                {
                  out.write(current, 0, current.length());
                  out.newLine();
                }
                current = "# JAAS login configuration file location";
                out.write(current, 0, current.length());
                out.newLine();
                current = "login.config.url.1=file:${java.home}/lib/security/config.pd";
                out.write(current, 0, current.length());
                out.newLine();
                out.newLine();
                current = "# JAAS authorization policy file location";
                out.write(current, 0, current.length());
                out.newLine();
                current = "auth.policy.url.1=file:${java.home}/lib/security/jaas.policy";
                out.write(current, 0, current.length());
                out.newLine();
                in.close();
                out.close();

            } catch (Exception e){
                System.out.println("Exception occurred updating up java.security");
            }

            // back up the java.policy file
            try {

                filename = jre + fs + "lib" + fs + "security" + fs + "java.policy";
                System.out.println("Backing up file " + filename);

                in = new BufferedReader(new FileReader(filename));
                out = new BufferedWriter(new FileWriter(filename + ".orig"));

                String current = null;
                while ((current = in.readLine()) != null)
                {
                  out.write(current, 0, current.length());
                  out.newLine();
                }
                in.close();
                out.close();

            } catch (Exception e){
                System.out.println("Exception occurred backing up java.policy");
            }

            // update the java.policy file
            try {

                System.out.println("Updating file " + filename);

                in = new BufferedReader(new FileReader(filename + ".orig"));
                out = new BufferedWriter(new FileWriter(filename));

                String nextBack = in.readLine();
                String previous = in.readLine();
                String current = null;
                while ((current = in.readLine()) != null)
                {
                  out.write(nextBack, 0, nextBack.length());
                  out.newLine();
                  nextBack = previous;
                  previous = current;
                }
                current = "        // for PDJaasDemo";
                out.write(current, 0, current.length());
                out.newLine();
                current = "	permission javax.security.auth.AuthPermission \"createLoginContext\";";
                out.write(current, 0, current.length());
                out.newLine();
                current = "	permission javax.security.auth.AuthPermission \"doAs\";";
                out.write(current, 0, current.length());
                out.newLine();
                current = "	permission javax.security.auth.AuthPermission \"doAsPrivileged\";";
                out.write(current, 0, current.length());
                out.newLine();
                current = "	permission javax.security.auth.AuthPermission \"modifyPrincipals\";";
                out.write(current, 0, current.length());
                out.newLine();
                current = "	permission javax.security.auth.AuthPermission \"getSubject\";";
                out.write(current, 0, current.length());
                out.newLine();
                current = "	permission javax.security.auth.AuthPermission \"createPDPrincipal\";";
                out.write(current, 0, current.length());
                out.newLine();
                out.write(nextBack, 0, nextBack.length());
                out.newLine();
                out.write(previous, 0, previous.length());
                out.newLine();
                in.close();
                out.close();

            } catch (Exception e){
                System.out.println("Exception occurred updating up java.policy");
            }

            // create the config.pd file
            try {

                filename = jre + fs + "lib" + fs + "security" + fs + "config.pd";
                System.out.println("Creating file " + filename);

                out = new BufferedWriter(new FileWriter(filename));

                String current = "//// config.pd: Login configuration file for PDLoginModule";
                out.write(current, 0, current.length());
                out.newLine();
                out.newLine();
                current = "pd-debug {";
                out.write(current, 0, current.length());
                out.newLine();
                current = "        com.tivoli.pd.jazn.PDLoginModule required debug=true;";
                out.write(current, 0, current.length());
                out.newLine();
                current = "};";
                out.write(current, 0, current.length());
                out.newLine();
                out.newLine();
                current = "pd {";
                out.write(current, 0, current.length());
                out.newLine();
                current = "        com.tivoli.pd.jazn.PDLoginModule required;";
                out.write(current, 0, current.length());
                out.newLine();
                current = "};";
                out.write(current, 0, current.length());
                out.newLine();
                out.newLine();
                current = "pd-nopass {";
                out.write(current, 0, current.length());
                out.newLine();
                current = "        com.tivoli.pd.jazn.PDLoginModule required nameOnly=true;";
                out.write(current, 0, current.length());
                out.newLine();
                current = "};";
                out.write(current, 0, current.length());
                out.newLine();
                out.close();

            } catch (Exception e){
                System.out.println("Exception occurred creating config.pd");
            }

            // create the jaas.policy file
            try {

                filename = jre + fs + "lib" + fs + "security" + fs + "jaas.policy";
                System.out.println("Creating file " + filename);

                out = new BufferedWriter(new FileWriter(filename));

                String current = "grant Principal com.tivoli.pd.jazn.PDPrincipal \"*\" {";
                out.write(current, 0, current.length());
                out.newLine();
                current = "	permission com.tivoli.pd.jazn.PDPermission \"ignoreme\", \"a\";";
                out.write(current, 0, current.length());
                out.newLine();
                current = "};";
                out.write(current, 0, current.length());
                out.newLine();
                out.close();

            } catch (Exception e){
                System.out.println("Exception occurred creating jaas.policy");
            }

        }

    }

    /**
     * Prints out any input PDMessages and clears the PDMessages.
     *
     * <p>
     * @param msgs   PDMessages to process.  If there are any PDMessages, they
     *               are printed and then cleared.
     */
    private static void processMsgs(PDMessages msgs)
    {
       if (msgs.size() > 0)
       {
          System.out.println("TamLogin PDAdmin msgs: " + msgs + "\n");
          msgs.clear();
       }
    }

    /**
     * Handle a PDException.
     *
     * <p>
     * @param e     PDException to handle. Any message text contained in the
     *              PDException is printed.  If there are PDMessages contained,
     *              the stack trace is printed for the cause of the PDException.
     */
    private static void handlePDException(Exception e)
    {
       PDException pd = (PDException)e;
       PDMessages msgs = pd.getMessages();
       if (msgs != null)
       {
          Iterator pdi = msgs.iterator();
          PDMessage msg = null;

          while (pdi.hasNext())
          {
             msg = (PDMessage)pdi.next();
             System.out.println(msg.getMsgText());
          }
       }
       else
       {
           Throwable t = pd.getCause();
           if (t != null)
           {
              System.out.println("*** Underlying Java Exception ***");
              t.printStackTrace();
           }
       }
    }
}
